package repository

import (
	"context"
	"errors"
	"log/slog"

	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgxpool"
)

var (
	ErrNotFound error = errors.New("todo cannot be found")
)

type TodoRow struct {
	Id   int64
	Name string
	Done bool
}

func NewTodoRow(name string, done bool) TodoRow {
	return TodoRow{Name: name, Done: done}
}

func (t TodoRow) Equal(todo TodoRow) bool {
	return t.Id == todo.Id && t.Name == todo.Name && t.Done == todo.Done
}

type TodoRepository struct {
	logger *slog.Logger
	db     *pgxpool.Pool
}

func NewTodoRepository(logger *slog.Logger, db *pgxpool.Pool) *TodoRepository {
	return &TodoRepository{
		logger: logger,
		db:     db,
	}
}

func (r *TodoRepository) GetById(ctx context.Context, id int64) (TodoRow, error) {
	todo := TodoRow{}

	err := r.db.QueryRow(ctx, "SELECT * FROM todo WHERE id = $1;", id).Scan(&todo.Id, &todo.Name, &todo.Done)

	if err != nil {
		if err == pgx.ErrNoRows {
			return todo, ErrNotFound
		}

		r.logger.ErrorContext(ctx, err.Error())
		return todo, err
	}

	return todo, nil
}

func (r *TodoRepository) Create(ctx context.Context, todo TodoRow) (TodoRow, error) {
	result := r.db.QueryRow(ctx, "INSERT INTO todo (name, done) VALUES ($1, $2) RETURNING id;", todo.Name, todo.Done)
	err := result.Scan(&todo.Id)

	if err != nil {
		r.logger.ErrorContext(ctx, err.Error())
		return TodoRow{}, err
	}

	return todo, nil
}

func (r *TodoRepository) Update(ctx context.Context, todo TodoRow) error {
	result, err := r.db.Exec(ctx, "UPDATE todo SET name = $1, done = $2 WHERE id = $3;", todo.Name, todo.Done, todo.Id)

	if err != nil {
		r.logger.ErrorContext(ctx, err.Error())
		return err
	}

	rowsAffected := result.RowsAffected()

	if rowsAffected == 0 {
		return ErrNotFound
	}

	return nil
}

func (r *TodoRepository) Delete(ctx context.Context, id int64) error {
	result, err := r.db.Exec(ctx, "DELETE FROM todo WHERE id = $1;", id)

	if err != nil {
		r.logger.ErrorContext(ctx, err.Error())
		return err
	}

	rowsAffected := result.RowsAffected()

	if rowsAffected == 0 {
		return ErrNotFound
	}

	return nil
}
